/* data structure definitions */
#define MAX_ENTS	100
#define MAX_LEN	80
#define USE_NOT   0
#define USE_YES   1
#define USE_DEL   2
/*
 * Generic Structure for data entries
 *
 * This structure is used to contain the actual data
 * Each type of data organization will make use of
 * this to store the data
 */
struct Entry { 		/* generic entry */
   int use;				/* in use flag */
	int key;				/* numeric key for entry */
   char desc[MAX_LEN];	/* description of entry */
};

/*
 * Hash table Data Structure
 *
 * This structure is used to store data using hashing
 *
 * A hash table is created for each possible hash value
 *
 * In the current implementation, there are 10 possible values
 *
 */
struct HashTable {	/* hash table entry */
	int hash_val;		/* hash value */
   struct Entry entry[MAX_ENTS];	/* array of entries */
};

/*
 * Linked List
 *
 * A Linked List consists of two different data structures
 *
 * A Link Entry, which contains the actual data
 * and a Linked List, which consists of a series of links
 */
struct LinkEntry {		/* link list entry */
	int next;				/* location of next entry */
   int prev;				/* location of previous entry */
   struct Entry entry;	/* actual data within the list */
};

struct LinkedList {	/* linked list of entries */
	int head;			/* location of first entry in list */
   struct LinkEntry list[MAX_ENTS];	/* actual entries */
};

/*
 *			**** Queue ****
 *
 * This structure is used to create a Queue
 * A Queue is a data structure where entries are added to
 * one end and removed from the other end.
 */
struct Queue {
	int head;			/* loc. of front of queue (-1 for empty) */
   int tail;			/* loc. of end of queue (-1 for empty) */
   int num_ents;		/* number of entries in queue */
   struct Entry Entry[MAX_ENTS];
};

/*
 * This structure is used to form a table used to
 * map a use flag into a string containing a
 * description of the flag
 */
struct UseDesc { /* descriptions of use flag */
	int use;			/* use flag */
   char *desc;		/* description */
};

/* function prototypes for each major routine */
void init_array(void);
int do_array(void);
void init_hash_table(void);
int do_hash_table(void);
void init_linked_list(void);
int do_linked_list(void);
void init_queue(void);
int do_queue(void);
void init_stack(void);
int do_stack(void);
char *get_use_desc(int use);

