/*
 *				**** STACK ****
 *
 * This file contains the routines related to stacks
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "datademo.h"

static struct Entry Stack[MAX_ENTS];	/* stack of entries */
static int num_ents = 0;					/* number of entries in stack */

/* function prototypes */
void push_stack(void);
void pop_stack(void);
void list_stack(void);
void search_stack(void);
void display_stack(int ent);

int
do_stack(void)
{
	char buf[BUFSIZ];

   while (1)
   {
   	printf("\nOptions are:\n");
      printf("A. Add entries to stack\n");
      printf("D. Delete entries from stack\n");
      printf("L. List stack\n");
      printf("S. Search stack\n");
      printf("M. Return to Main Menu\n");
      printf("Q. Quit\n");
      printf("Enter option: ");
      gets(buf);
      switch(buf[0])
      {
      case 'a':
      case 'A':
      	push_stack();
         break;
      case 'd':
      case 'D':
      	pop_stack();
         break;
      case 'l':
      case 'L':
      	list_stack();
         break;
      case 's':
      case 'S':
      	search_stack();
         break;
      case 'm':
      case 'M':
      	return 1;	/* keep running */

      case 'q':
      case 'Q':
      	return 0;	/* don't keep running */

      default:
      	printf("Invalid option - try again!\n");
      }
   }
}

void
push_stack(void)
{ /* add an entry to the stack */
	char buf[BUFSIZ];
   int key;

   printf("There are %d entries currently in use\n", num_ents);

   /* make sure stack isn't full */
   if (num_ents >= MAX_ENTS)
   {
   	printf("Stack is full.  Delete some entries first\n");
      return;
   }

   /* ready to add to end of stack */
   printf("Enter key: ");
   gets(buf);
   key = atoi(buf);		/* set key */
   do
   {
	   printf("Enter description: ");
	   gets(buf);
	   if (strlen(buf) > MAX_LEN)
      {
      	printf("Name too long.  Try again? (y or n): ");
         gets(buf);
         if (buf[0] == 'n') return;
      }
   } while(strlen(buf) >= MAX_LEN);

   printf("Adding new entry in location %d\n", num_ents);
   Stack[num_ents].key = key;
   Stack[num_ents].use = USE_YES;
   strcpy(Stack[num_ents].desc, buf);
   num_ents++;		/* one more entry in stack */
   printf("There are now %d entries in use\n", num_ents);
}

void
pop_stack(void)
{
	char buf[BUFSIZ];
   int ent;

   printf("There are currently %d entries in use\n", num_ents);

   ent = num_ents - 1;		/* entry on top of stack */
   printf("Entry to be removed:\n");
   display_stack(ent);
   printf("OK? (y or n): ");
   gets(buf);
   if (buf[0] != 'y') return;		/* don't remove */

   num_ents--;		/* one less entry in stack */
   printf("There are now %d entries in use\n", num_ents);
}

void
list_stack(void)
{
	int i;

   printf("%d entries in table ...\n", num_ents);

   for (i=num_ents-1; i >= 0; i--) display_stack(i);
}

void
display_stack(int ent)
{
	printf("Entry %d, key %d, desc: %s\n",
   	ent, Stack[ent].key, Stack[ent].desc);
}

void
search_stack(void)
{
	char buf[BUFSIZ];
   int key, i, num_matches = 0;

   printf("Enter key to search for: ");
   gets(buf);
   key = atoi(buf);

   printf("Searching for key: %d\n", key);

   for (i=num_ents-1; i >= 0; i--)
   {
   	if (key == Stack[i].key)
      { /* key match found */
      	display_stack(i);
         num_matches++;
      }
   }
   printf("%d matches found\n", num_matches);
}

void
init_stack(void)
{
	int i;

   /*
    * Initialize Stack
    *
    * set number of entries to zero
    * clear all entries in the stack
    */
	num_ents = 0;		/* how many entries in stack */
	for (i=0; i < MAX_ENTS; i++)
   { /* clear stack entries */
   	Stack[i].use = USE_NOT;		/* not in use */
      Stack[i].key = 0;					/* no key */
      Stack[i].desc[0] = '\0';		/* clear description */
   }
}

